/*
 * Connect3D - Reconstruction of water-tight triangulated meshes from unstructured point clouds
 *
 * please credit the following article: Stefan Ohrhallinger, Sudhir Mudur and Michael Wimmer:
 * 'Minimizing Edge Length to Connect Sparsely Sampled Unstructured Point Sets',
 * Shape Modeling International 2013, published in Computers & Graphics Journal, 2013.
 *
 * Copyright (C) 2013 Stefan Ohrhallinger, Daniel Prieler
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */
 
#pragma once


struct  VertexData
{
	std::unique_ptr<Vector3D []> vertices;
	size_t numVertices;
};


class GeometryLoader
{

public:

	static  VertexData *loadFile(const std::string &fileName);

private:
	GeometryLoader() {}
	~GeometryLoader() {}

	static  VertexData *readPlyFile(const std::string &path);
	static  VertexData *readPtsFile(const std::string &path);

};

class GeometryWriter
{

public:

	static bool writeOFFFile(const std::string &fileName, const std::vector<Triangle *> &triangles, const Dt *dTriangulation, const Vector3D *vertexData);
	static bool testWrite(const std::string &fileName);

private:
	GeometryWriter() {}
	~GeometryWriter() {}

};
